// $Id: Anon.h,v 1.2 2002/10/26 04:15:44 vern Exp $

// The IP address anonymization code is largely based on (and sometimes
// directly copied from) Eddie Kohler's ipsumdump-1.20 code, per:
//
//	http://www.icir.org/kohler/ipsumdump/
//
// ipsumdump, in turn, takes some of its code from tcpdpriv:
//
//	http://ita.ee.lbl.gov/html/contrib/tcpdpriv.html

#ifndef anon_h
#define anon_h

#include <vector>
#include "net_util.h"

enum anonymization_method_t {
	// FULLY_RANDOM,
	PREFIX_PRESERVING_A50,
};

typedef uint32 ipaddr32_t;

// NOTE: all addresses in parameters of *public* functions are in
// network order.

class AnonymizeIPAddr {
public:
	AnonymizeIPAddr(anonymization_method_t method, int user_seed = 0);
	~AnonymizeIPAddr();

	ipaddr32_t Anonymize(ipaddr32_t addr);

	// Keep the specified prefix unchanged.
	int PreservePrefix(ipaddr32_t input, int num_bits);
	int PreserveNet(ipaddr32_t input);

protected:
	struct Node {
		ipaddr32_t input;
		ipaddr32_t output;
		Node* child[2];
	};

	int method;
	int before_anonymization;
	int new_mapping;

	// The root of prefix preserving mapping tree.
	Node* root;

	// A node pool for new_node.
	Node* next_free_node;
	vector<Node*> blocks;

	// for 0.0.0.0 and 255.255.255.255.
	Node special_nodes[2];

	void init();

	Node* new_node();
	Node* new_node_block();
	void free_node(Node*);

	ipaddr32_t make_output(ipaddr32_t, int) const;
	Node* make_peer(ipaddr32_t, Node*);
	Node* find_node(ipaddr32_t);
};

extern AnonymizeIPAddr* ip_anonymizer;		// the global IP anonymizer

// ### Ruoming, what is the following for?
#define LOG_ANONYMIZATION_MAPPING
void log_anonymization_mapping(ipaddr32_t input, ipaddr32_t output);

#endif
