# $Id: http-reply.bro,v 1.12 2002/11/16 00:37:53 vern Exp $

@load http-request

redef capture_filter +=
	"tcp src port 80 or tcp src port 8080 or tcp src port 8000";

redef process_HTTP_replies = T;

event http_reply(c: connection, version: string, code: count, reason: string)
	{
	local s = lookup_http_request_stream(c);
	local msg = s$next_reply;

	init_http_message(msg);

	msg$initiated = T;
	msg$code = code;
	msg$reason = reason;
	}

function http_request_done(c: connection, body_length: count): any
	{
	local s = lookup_http_request_stream(c);
	local msg = s$next_request;
	msg$initiated = F;
	}

function http_reply_done(c: connection, body_length: count): any
	{
	local s = lookup_http_request_stream(c);
	local msg = s$next_reply;
	local req: string;
	local log_it: bool;

	if ( s$num_pending_requests == 0 )
		{
		# Weird - reply w/o request - perhaps due to cold start?
		req = "<unknown request>";
		log_it = F;
		}
	else
		{
		local r = s$requests[s$first_pending_request];

		# Remove pending request.
		delete s$requests[s$first_pending_request];
		--s$num_pending_requests;
		++s$first_pending_request;

		req = fmt("%s %s", r$method, r$URI);
		log_it = r$log_it;
		}

	local size = body_length > 0 ? fmt(" [%d]", body_length) : "";
	local req_rep =
		fmt("%s (%d%s %s)", req, msg$code, size, msg$reason);

	if ( log_it )
		ALERT([$alert=HTTP_SensitiveURI, $conn=c,
			$msg=fmt("%s %s: %s",
				id_string(c$id), c$addl, req_rep)]);

	print http_log, fmt("%.6f %s %s", network_time(), s$id, req_rep);

	msg$initiated = F;
	}

event http_message_done(c: connection, is_orig: bool, body_length: count)
	{
	if ( is_orig )
		http_request_done(c, body_length);
	else
		http_reply_done(c, body_length);
	}
