# $Id: http-rewriter.bro,v 1.6 2002/10/26 03:50:11 vern Exp $

# We can't do HTTP rewriting unless we process everything in the connection.
@load http-reply
@load http-entity

redef rewriting_http_trace = T;
redef http_entity_data_delivery_size = 4096;

event http_request(c: connection, method: string, URI: string, version: string)
	{
	rewrite_http_request(c, method, URI, version);
	}

event http_reply(c: connection, version: string, code: count, reason: string)
	{
	rewrite_http_reply(c, version, code, reason);
	}

event http_header(c: connection, is_orig: bool, name: string, value: string)
	{
	# Only rewrite top-level headers.
	local s = lookup_http_request_stream(c);
	local msg = get_http_message(s, is_orig);

	if ( msg$entity_level == 1 )
		{
		if ( name == "CONTENT-LENGTH" )
			rewrite_http_header(c, is_orig,
					"X-Original-Content-Length", value);

		else if ( name == "TRANSFER-ENCODING" )
			rewrite_http_header(c, is_orig,
					"X-Original-Transfer-Encoding", value);

		else
			rewrite_http_header(c, is_orig, name, value);
		}
	}

event http_message_done(c: connection, is_orig: bool, body_length: count)
	{
	local s = lookup_http_request_stream(c);
	local msg = get_http_message(s, is_orig);
	local data_length = 0;

	if ( ! is_orig || body_length > 0 )
		{
		if ( include_HTTP_abstract )
			data_length = byte_len(msg$abstract);
		rewrite_http_header(c, is_orig, "Content-Length", fmt(" %d", data_length));
		}

	if ( msg$data_length > 0 && body_length != msg$data_length )
		log fmt("length mismatch: %s %d %d", id_string(c$id), body_length, msg$data_length);

	rewrite_http_header(c, is_orig, "X-Actual-Data-Length",
				fmt(" %d; content-length=%d",
					body_length, msg$content_length));
	rewrite_http_data(c, is_orig, "\r\n");

	if ( data_length > 0 )
		rewrite_http_data(c, is_orig, msg$abstract);

	# Push the packet so that it gets to the trace before RST does.
	rewrite_push_packet(c, is_orig);
	}
