# $Id: ident.bro,v 1.15 2002/10/20 02:35:20 vern Exp $

@load alert
@load active
@load hot-ids

redef enum Alert += {
	IdentSensitiveID,	# sensitive username in Ident lookup
};

redef capture_filter += "tcp port 113";

global hot_ident_ids = { always_hot_ids, } &redef;
global hot_ident_exceptions = { "uucp", "nuucp", "daemon", } &redef;
global public_ident_user_ids = { "root", } &redef;
global public_ident_systems = { "UNIX", } &redef;

global pending_ident_requests: set[addr, port, addr, port, port, port];

global rewrite_ident_trace: bool;

event bro_init()
	{
	rewrite_ident_trace = rewriting_trace();
	}

event ident_request(c: connection, lport: port, rport: port)
	{
	local id = c$id;
	add pending_ident_requests[id$orig_h, id$orig_p, id$resp_h, id$resp_p, lport, rport];
	if ( rewrite_ident_trace )
		rewrite_ident_request(c, lport, rport);
	}

function add_ident_tag(c: connection, lport: port, rport: port, tag: string)
: connection
	{
	local id = c$id;
	if ( [id$orig_h, id$orig_p, id$resp_h, id$resp_p, lport, rport] in
	     pending_ident_requests )
		delete pending_ident_requests[id$orig_h, id$orig_p, id$resp_h, id$resp_p, lport, rport];
	else
		tag = fmt("orphan-%s", tag);

	local c_orig =
		[id$resp_h, rport, id$orig_h, lport] in active_conn ?
			active_conn[id$resp_h, rport, id$orig_h, lport] : c;

	append_addl(c_orig, tag);

	return c_orig;
	}

function public_ident_user(id: string): bool
	{
	return id in public_ident_user_ids;
	}

function public_system(system: string): bool
	{
	return system in public_ident_systems;
	}

event ident_reply(c: connection, lport: port, rport: port,
		user_id: string, system: string)
	{
	local c_orig = add_ident_tag(c, lport, rport, fmt("ident/%s", user_id));

	if ( user_id in hot_ident_ids && user_id !in hot_ident_exceptions )
		{
		++c_orig$hot;
		ALERT([$alert=IdentSensitiveID, $conn=c,
			$msg=fmt("%s hot ident: %s",
				id_string(c_orig$id), c_orig$addl)]);
		}

	if ( rewrite_ident_trace )
		rewrite_ident_reply(c, lport, rport, 
				public_system(system) ? system : "OTHER", 
				public_ident_user(user_id) ? user_id : "private user");
	}

event ident_error(c: connection, lport: port, rport: port, line: string)
	{
	add_ident_tag(c, lport, rport, fmt("iderr/%s", line));
	if ( rewrite_ident_trace )
		rewrite_ident_error(c, lport, rport, line);
	}

# event connection_content_EOF(c: connection, is_orig: bool)
#	{
#	if ( rewrite_ident_trace )
#		rewrite_ident_EOF(c, is_orig);
#	}
