%{
/* $Id: rule-parse.y,v 1.4 2002/10/11 02:28:43 leres Exp $ */

#include <stdio.h>
#include "RuleMatcher.h"

Rule* current_rule = 0;
%}

%token TOK_AND
%token TOK_BOTH
%token TOK_COMP
%token TOK_ESTABLISHED
%token TOK_EVAL
%token TOK_EVENT
%token TOK_HEADER
%token TOK_ICMP
%token TOK_ID
%token TOK_IDENT
%token TOK_INT
%token TOK_IP
%token TOK_IPOPTIONS
%token TOK_IPOPTIONSYM
%token TOK_LSRR
%token TOK_LSRRE
%token TOK_ORIGINATOR
%token TOK_PATTERN
%token TOK_PATTERNTYPE
%token TOK_PAYLOADSIZE
%token TOK_PROT
%token TOK_REQUIRESRULE
%token TOK_REQUIRESRULEOPPOSITE
%token TOK_RESPONDER
%token TOK_RIGHT
%token TOK_RR
%token TOK_RULE
%token TOK_SAMEIP
%token TOK_TCPSTATE
%token TOK_SSRR
%token TOK_STATELESS
%token TOK_STRING
%token TOK_TCP
%token TOK_TCPSTATESYM
%token TOK_UDP

%left  TOK_AND

%type <str> TOK_STRING TOK_IDENT TOK_PATTERN
%type <val> TOK_INT TOK_TCPSTATESYM tcpstate_list TOK_IPOPTIONSYM
%type <val> ipoption_list TOK_COMP
%type <rule> rule
%type <hdr_test> hdr_expr
%type <range> range rangeopt
%type <vallist> value_list
%type <mval> TOK_IP value
%type <prot> TOK_PROT
%type <ptype> TOK_PATTERNTYPE

%union {
	Rule* rule;
	RuleHdrTest* hdr_test;
	maskedvalue_list* vallist;

	int val;
	char* str;
	MaskedValue mval;
	RuleHdrTest::Prot prot;
	Range range;
	Rule::PatternType ptype;
}

%%

rule_list:
		rule_list rule
			{ rule_matcher->AddRule($2); }
	|
	;

rule:
		TOK_RULE TOK_IDENT
			{ current_rule = new Rule(yylval.str); }
		'{' rule_attr_list '}'
			{ $$ = current_rule; }
	;

rule_attr_list:
		rule_attr_list rule_attr
	|
	;

rule_attr:
		TOK_EVAL TOK_IDENT
			{ current_rule->AddCondition(new RuleConditionEval($2)); }

	|	TOK_HEADER hdr_expr
			{ current_rule->AddHdrTest($2); }

	|	TOK_IPOPTIONS ipoption_list
			{ current_rule->AddCondition(new RuleConditionIPOptions($2)); }

	|	TOK_EVENT TOK_STRING
			{ current_rule->AddAction(new RuleActionEvent( $2 )); }

	|	TOK_PATTERNTYPE TOK_PATTERN
			{ current_rule->AddPattern($2, $1); }

	|	TOK_PATTERNTYPE '[' rangeopt ']' TOK_PATTERN
			{
			if ( $3.offset > 0 )
				warn("Offsets are currently ignored for patterns");
			current_rule->AddPattern($5, $1, 0, $3.len);
			}

	|	TOK_PAYLOADSIZE TOK_COMP TOK_INT
			{
			current_rule->AddCondition(
				new RuleConditionPayloadSize($3, (RuleConditionPayloadSize::Comp) ($2)));
			}

	|	TOK_REQUIRESRULE TOK_IDENT
			{ current_rule->AddRequires($2, 0, 0); }

	|	TOK_REQUIRESRULE '!' TOK_IDENT
			{ current_rule->AddRequires($3, 0, 1); }

	|	TOK_REQUIRESRULEOPPOSITE TOK_IDENT
			{ current_rule->AddRequires($2, 1, 0); }

	|	TOK_REQUIRESRULEOPPOSITE '!' TOK_IDENT
			{ current_rule->AddRequires($3, 1, 1); }

	|	TOK_SAMEIP
			{ current_rule->AddCondition(new RuleConditionSameIP()); }

	|	TOK_TCPSTATE tcpstate_list
			{ current_rule->AddCondition(new RuleConditionTCPState($2)); }
	;

hdr_expr:
		TOK_PROT '[' range ']' '&' TOK_INT TOK_COMP value
			{
			maskedvalue_list* vallist = new maskedvalue_list;
			MaskedValue* val = new MaskedValue();

			val->val = $8.val;
			val->mask = $6;
			vallist->append(val);

			$$ = new RuleHdrTest($1, $3.offset, $3.len,
					(RuleHdrTest::Comp) $7, vallist);
			}

	|	TOK_PROT '[' range ']' TOK_COMP value_list
			{
			$$ = new RuleHdrTest($1, $3.offset, $3.len,
						(RuleHdrTest::Comp) $5, $6);
			}
	;

value_list:
		value_list ',' value
			{ $1->append(new MaskedValue($3)); $$ = $1; }
	|	value
			{
			$$ = new maskedvalue_list();
			$$->append(new MaskedValue($1));
			}
	;

value:
		TOK_INT
			{ $$.val = $1; $$.mask = 0xffffffff; }
	|	TOK_IP
	;

rangeopt:
		range
			{ $$ = $1; }
	|	':' TOK_INT
			{ $$.offset = 0; $$.len = $2; }
	|	TOK_INT ':'
			{ $$.offset = $1; $$.len = UINT_MAX; }
	;

range:
		TOK_INT
			{ $$.offset = $1; $$.len = 1; }
	|	TOK_INT ':' TOK_INT
			{ $$.offset = $1; $$.len = $3; }
	;

ipoption_list:
		ipoption_list ',' TOK_IPOPTIONSYM
			{ $$ = $1 | $3; }
	|	TOK_IPOPTIONSYM
			{ $$ = $1; }
	;

tcpstate_list:
		tcpstate_list ',' TOK_TCPSTATESYM
			{ $$ = $1 | $3; }
	|	TOK_TCPSTATESYM
			{ $$ = $1; }
	;

%%

void rules_error(const char* msg)
	{
	fprintf(stderr, "Error in rule (line %d): %s\n",
			rules_line_number+1, msg);
	rule_matcher->SetParseError();
	}

void rules_error_by_id(const char* id, const char* msg)
	{
	fprintf(stderr, "Error in rule %s: %s\n", id, msg);
	rule_matcher->SetParseError();
	}

int rules_wrap(void)
	{
	return 1;
	}
