/* $Id: rule-scan.l,v 1.6 2002/10/13 21:12:02 vern Exp $ */

%{
typedef unsigned int uint32;

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "RuleMatcher.h"
#include "rule-parse.h"

int rules_line_number = 0;
%}

WS	[ \t]+
D	[0-9]+
H	[0-9a-fA-F]+
STRING	\"([^\n\"]|\\\")*\"
ID	[0-9a-zA-Z_-]+
RE	\/([^/]|[^\\]\\\/)*\/

%option nounput

%%

#.*	/* eat comments */
{WS}	/* eat white space */

\n	++rules_line_number;

[!\]\[{}&:,]	return rules_text[0];

"->"	return TOK_RIGHT;
"<>"	return TOK_BOTH;

"<="	{ rules_lval.val = RuleHdrTest::LE; return TOK_COMP; }
">="	{ rules_lval.val = RuleHdrTest::GE; return TOK_COMP; }
"<"	{ rules_lval.val = RuleHdrTest::LT; return TOK_COMP; }
">"	{ rules_lval.val = RuleHdrTest::GT; return TOK_COMP; }
"="	{ rules_lval.val = RuleHdrTest::EQ; return TOK_COMP; }
"=="	{ rules_lval.val = RuleHdrTest::EQ; return TOK_COMP; }
"!="	{ rules_lval.val = RuleHdrTest::NE; return TOK_COMP; }

ip	{ rules_lval.val = RuleHdrTest::IP; return TOK_PROT; }
icmp	{ rules_lval.val = RuleHdrTest::ICMP; return TOK_PROT; }
tcp	{ rules_lval.val = RuleHdrTest::TCP; return TOK_PROT; }
udp	{ rules_lval.val = RuleHdrTest::UDP; return TOK_PROT; }

established	{
		rules_lval.val = RuleConditionTCPState::STATE_ESTABLISHED;
		return TOK_TCPSTATESYM;
		}

originator	{
		rules_lval.val = RuleConditionTCPState::STATE_ORIG;
		return TOK_TCPSTATESYM;
		}

responder	{
		rules_lval.val = RuleConditionTCPState::STATE_RESP;
		return TOK_TCPSTATESYM;
		}

stateless	{
		rules_lval.val = RuleConditionTCPState::STATE_STATELESS;
		return TOK_TCPSTATESYM;
		}

lsrr		{
		rules_lval.val = RuleConditionIPOptions::OPT_LSRR;
		return TOK_IPOPTIONSYM;
		}

lsrre		{
		rules_lval.val = RuleConditionIPOptions::OPT_LSRRE;
		return TOK_IPOPTIONSYM;
		}

rr		{
		rules_lval.val = RuleConditionIPOptions::OPT_RR;
		return TOK_IPOPTIONSYM;
		}

ssrr		{
		rules_lval.val = RuleConditionIPOptions::OPT_SSRR;
		return TOK_IPOPTIONSYM;
		}

and		return TOK_AND;
eval		return TOK_EVAL;
event		return TOK_EVENT;
header		return TOK_HEADER;
id		return TOK_ID;
ip-options	return TOK_IPOPTIONS;
msg		return TOK_EVENT; /* FIXME: Remove; keyword is only for backwards compatibility */
payload-size	return TOK_PAYLOADSIZE;
requires-rule	return TOK_REQUIRESRULE;
requires-rule-opposite	return TOK_REQUIRESRULEOPPOSITE;
rule		return TOK_RULE;
same-ip		return TOK_SAMEIP;
tcp-state	return TOK_TCPSTATE;

payload	{ rules_lval.val = Rule::PAYLOAD; return TOK_PATTERNTYPE; }
http	{ rules_lval.val = Rule::HTTP; return TOK_PATTERNTYPE; }
ftp	{ rules_lval.val = Rule::FTP; return TOK_PATTERNTYPE; }

{D}("."{D}){3}"/"{D}	{
	char* s = strchr(yytext, '/');
	*s++ = '\0';

	rules_lval.mval.val = ntohl(inet_addr(yytext));
	rules_lval.mval.mask = ntohl((1 << (atoi(s))) - 1);

	return TOK_IP;
	}

{D}("."{D}){3}	{
	rules_lval.mval.val = ntohl(inet_addr(yytext));
	rules_lval.mval.mask = 0xffffffff;
	return TOK_IP;
	}

{D}	{
	rules_lval.val = (uint32) atoi(yytext);
	return TOK_INT;
	}

0x{H}	{
	rules_lval.val = (uint32) strtol(yytext, 0, 16);
	return TOK_INT;
	}


{STRING}	{
	*(yytext + strlen(yytext) - 1) = '\0';
	rules_lval.str = yytext + 1;
	return TOK_STRING;
	}

{ID}	{
	rules_lval.str = yytext;
	return TOK_IDENT;
	}

{RE}	{
	*(yytext + strlen(yytext) - 1) = '\0';
	rules_lval.str = yytext + 1;
	return TOK_PATTERN;
	}

.	rules_error("unrecognized character in input");

%%
